@@ -39,6 +39,7 @@ Author:
     Abiodun Solanke, Ph.D.
 """
 import os
+import json
 import pandas as pd
 from concurrent.futures import ThreadPoolExecutor
 import logging
@@ -49,8 +50,7 @@ import numpy as np
 import requests
 
 from netiob.settings import OREF0_API_SERVER_URL, MAX_WORKERS
-from netiob.utils.preprocessors import preprocess_user_data
-from netiob.utils.coreutils import get_pump_history, avg_basal_rate, datetime_to_zoned_iso, get_last_x_hr
+from netiob.utils.coreutils import get_pump_history, avg_basal_rate, datetime_to_zoned_iso, get_last_x_hr, make_json_serializable
 from netiob.utils.cgmdataprocessor import CGMDataProcessor
 
 # Configure module logger
@@ -104,14 +104,12 @@ def check_net_iob_server_is_live():
         
     except Exception as ex:
         # Log error with URL and exception details
-        logger.error(
-            f"NetIOB API is not available at {OREF0_API_SERVER_URL} | {ex.__str__()}"
-        )
+        logger.error(f"NetIOB API is not available at {OREF0_API_SERVER_URL} | {ex.__str__()}")
         # Re-raise exception to caller for handling
         raise
 
 
-def get_net_iob(pumphistory: list, profile: dict, clock: str, autosens: dict, pumphistory24: list=None) -> dict:
+def get_net_iob(pumphistory: list, profile: dict, clock: str, autosens: dict=None, pumphistory24: list=None) -> dict:
     """
     Calculate net insulin on board via OpenAPS /iob API endpoint.
     
@@ -188,14 +186,12 @@ def get_net_iob(pumphistory: list, profile: dict, clock: str, autosens: dict, pu
         
     except Exception as ex:
         # Log error with API URL and exception details
-        logger.error(
-            f"NetIOB API is not available {OREF0_API_SERVER_URL} | {ex.__str__()}"
-        )
+        logger.error(f"NetIOB API is not available {OREF0_API_SERVER_URL} | {ex.__str__()}")
         # Re-raise exception to caller for handling
         raise
 
 
-def calculate_net_iob(basal_df: pd.DataFrame, bolus_df: pd.DataFrame, carbs_df: pd.DataFrame, cgm_df: pd.DataFrame, profile: pd.DataFrame=None, autosens: dict=None) -> list:
+def calculate_net_iob(basal_df: pd.DataFrame, bolus_df: pd.DataFrame, cgm_df: pd.DataFrame, profile: pd.DataFrame=None, autosens: dict=None) -> list:
     """
     Calculate net IOB time series across full insulin history with parallel processing.
     
@@ -220,10 +216,6 @@ def calculate_net_iob(basal_df: pd.DataFrame, bolus_df: pd.DataFrame, carbs_df:
             temporal data to establish calculation range.
         bolus_df (pd.DataFrame): Bolus insulin delivery records with columns for
             timestamps and insulin amounts (units). Includes manual and automated boluses.
-        carbs_df (pd.DataFrame): Carbohydrate intake records with columns for timestamps
-            and carbohydrate amounts (grams). Used for preprocessing context.
-        cgm_df (pd.DataFrame): Continuous glucose monitoring readings with columns for
-            timestamps and glucose values (mg/dL or mmol/L). Used for preprocessing.
         profile (pd.DataFrame, optional): User profile settings containing DIA, basal
             schedule, ISF, carb ratios, and target ranges. If None, calculates average
             basal profile from processed insulin data. Default is None.
@@ -263,11 +255,11 @@ def calculate_net_iob(basal_df: pd.DataFrame, bolus_df: pd.DataFrame, carbs_df:
     
     Example:
         >>> # Basic usage with DataFrames
-        >>> iob_series = calculate_net_iob(basal_df=basal_data, bolus_df=bolus_data, carbs_df=carb_data, cgm_df=glucose_data)
+        >>> iob_series = calculate_net_iob(basal_df=basal_data, bolus_df=bolus_data)
         >>> print(f"Calculated {len(iob_series)} IOB time points")
         >>> 
         >>> # With profile and autosens
-        >>> iob_series = calculate_net_iob(basal_df=basal_data, bolus_df=bolus_data, carbs_df=carb_data, cgm_df=glucose_data, profile=user_profile_df, autosens={'ratio': 1.1, 'newisf': 45})
+        >>> iob_series = calculate_net_iob(basal_df=basal_data, bolus_df=bolus_data, profile=user_profile_df, autosens={'ratio': 1.1, 'newisf': 45})
         >>> 
         >>> # Extract IOB values for analysis or plotting
         >>> times = [pd.to_datetime(pt['time']) for pt in iob_series]
@@ -280,7 +272,7 @@ def calculate_net_iob(basal_df: pd.DataFrame, bolus_df: pd.DataFrame, carbs_df:
         >>> iob_df.set_index('time', inplace=True)
     """
     # Validate input types
-    if not all(isinstance(df, pd.DataFrame) for df in [basal_df, bolus_df, carbs_df, cgm_df]):
+    if not all(isinstance(df, pd.DataFrame) for df in [basal_df, bolus_df]):
         raise TypeError(
             "All input data (basal_df, bolus_df, carbs_df, cgm_df) must be pandas DataFrames"
         )
@@ -292,16 +284,16 @@ def calculate_net_iob(basal_df: pd.DataFrame, bolus_df: pd.DataFrame, carbs_df:
         raise TypeError("autosens must be a dictionary or None")
     
     # Validate DataFrames are not empty
-    if any(df.empty for df in [basal_df, bolus_df, carbs_df, cgm_df]):
+    if any(df.empty for df in [basal_df, bolus_df]):
         raise ValueError(
-            "Input DataFrames (basal_df, bolus_df, carbs_df, cgm_df) cannot be empty"
+            "Input DataFrames (basal_df, bolus_df) cannot be empty"
         )
     
     # Initialize CGMDataProcessor to preprocess all input data
     # This handles normalization, cleaning, and extraction of core structures
     try:
         logger.info("Initializing CGMDataProcessor for insulin data preprocessing")
-        dataprocessor = CGMDataProcessor(basal_df, bolus_df, carbs_df, cgm_df, profile_df=profile)
+        dataprocessor = CGMDataProcessor(basal_df=basal_df, bolus_df=bolus_df, cgm_df=cgm_df, profile_df=profile)
         
         # Extract preprocessed data from processor
         insulin_df = dataprocessor.proc_insulin
@@ -340,9 +332,7 @@ def calculate_net_iob(basal_df: pd.DataFrame, bolus_df: pd.DataFrame, carbs_df:
         logger.error(f"API availability check failed: {e}")
         raise
     
-    logger.info(
-        f"NetIOB API is available at {OREF0_API_SERVER_URL}, starting IOB calculations"
-    )
+    logger.info(f"NetIOB API is available at {OREF0_API_SERVER_URL}, starting IOB calculations")
 
     # Preprocess insulin data: fill NaN and round to 2 decimals
     # This ensures consistent numeric values for calculations
@@ -402,7 +392,7 @@ def calculate_net_iob(basal_df: pd.DataFrame, bolus_df: pd.DataFrame, carbs_df:
 
         # Build pump history structure from insulin records
         # This converts DataFrame rows to OpenAPS pump event format
-        pumphistory, _ = get_pump_history(last_xhr_records)
+        pumphistory = get_pump_history(insulin_data, last_xhr_records)[0]
 
         # Skip this time point if no pump history available
         # This can occur early in the time series before sufficient data accumulates
