@@ -544,21 +544,44 @@ def get_autosens(pumphistory: list, profile: dict, basal_profile: list, glucose_
         ...     # ratio = 1.2 means 20% more insulin sensitive than profile
     """
     # Assemble autosensitivity API inputs
+
+    # Basic validation
+    if not glucose_data or len(glucose_data) < 9:
+        print(f"WARNING: Insufficient glucose data ({len(glucose_data)} points)")
+        return {"ratio": 1.0, "newisf": profile.get("sens", 50)}
+    
     autosens_inputs = {
         "iob_inputs": {
-            "history": pumphistory,  # Insulin delivery history for IOB calculation
-            "profile": profile  # Profile for insulin action curves
+            "history": pumphistory,
+            "profile": profile
         },
-        "glucose_data": glucose_data,  # BG readings with rate-of-change
-        "basalprofile": basal_profile,  # Scheduled basal rates for deviation analysis
-        "temptargets": [],  # Temporary BG targets (none for now)
-        "carbs": [],  # Carb events (handled separately in meal data)
-        "retrospective": False  # Disable retrospective mode for real-time prediction
+        "glucose_data": glucose_data,
+        "basalprofile": basal_profile,
+        "temptargets": [],
+        "carbs": [],
+        "retrospective": False
     }
 
-    # Call OpenAPS /detect_sensitivity API
-    return call_api("/detect_sensitivity", autosens_inputs)
-
+    try:
+        detect_sens = call_api("/detect_sensitivity", autosens_inputs)
+        
+        if detect_sens is not None and isinstance(detect_sens, dict):
+            # Validate the response has valid ratio
+            ratio = detect_sens.get('ratio')
+            if ratio is not None and isinstance(ratio, (int, float)):
+                return detect_sens
+            else:
+                print(f"WARNING: Invalid ratio in response: {ratio}")
+        else:
+            print(f"WARNING: Invalid API response: {detect_sens}")
+            
+    except Exception as e:
+        print(f"ERROR calling detect_sensitivity: {e}")
+    
+    # Fallback to safe defaults
+    newisf = profile.get("sens", 50)
+    return {"ratio": 1.0, "newisf": newisf}
+    
 
 def get_profile_inputs(profile_df: pd.DataFrame, basal_df: pd.DataFrame) -> tuple:
     """
@@ -711,7 +734,7 @@ def get_last_x_hr(df: pd.DataFrame, time: str, pull_hr: int) -> pd.DataFrame:
         df (pd.DataFrame): Insulin dataframe with required column:
             - FADTC (str or datetime): Insulin delivery timestamps
         time (str or datetime): End timestamp for time window (most recent point).
-        pull_hr (int): Number of hours to look back from end timestamp.
+        pull_min (int): Number of minutes to look back from end timestamp.
             Should be >= DIA * 60 to capture all active insulin.
         
     Returns:
@@ -729,7 +752,7 @@ def get_last_x_hr(df: pd.DataFrame, time: str, pull_hr: int) -> pd.DataFrame:
         >>> insulin_data = get_last_x_hr(
         ...     df=insulin_df,
         ...     time='2025-11-03T12:00:00Z',
-        ...     pull_hr=24  # 6 hours
+        ...     pull_min=360  # 6 hours
         ... )
         >>> # Returns insulin from 06:00:00 to 12:00:00
     """
@@ -737,7 +760,7 @@ def get_last_x_hr(df: pd.DataFrame, time: str, pull_hr: int) -> pd.DataFrame:
     #ts = parse_iso_utc(time)
     
     # Calculate window start time (e.g., 6 hours before end)
-    start = ts - timedelta(hours=pull_hr)
+    start = time - timedelta(hours=pull_hr)
     
     # Parse timestamp column to datetime for comparison
     # Ensures all timestamps are timezone-aware UTC
@@ -897,7 +920,7 @@ def avg_basal_rate(insulin_data: pd.DataFrame) -> tuple:
     return basal_rate_structure_list, profile
 
 
-def get_pump_history(insulin_df: pd.DataFrame) -> tuple:
+def get_pump_history(insulin_df: pd.DataFrame, filtered_df: pd.DataFrame=None) -> tuple:
     """
     Build OpenAPS-compatible pump history from insulin dataframe.
     
@@ -910,7 +933,7 @@ def get_pump_history(insulin_df: pd.DataFrame) -> tuple:
     change is represented by paired TempBasal and TempBasalDuration events.
     
     Args:
-        insulin_df (pd.DataFrame): Filtered insulin data with columns:
+        insulin_df (pd.DataFrame): processed insulin data with columns:
             - FADTC (datetime): Delivery timestamps
             - FACAT (str): Category ('BASAL' or 'BOLUS')
             - FATEST (str): Test type ('BASAL INSULIN' or 'BOLUS INSULIN')
@@ -918,6 +941,8 @@ def get_pump_history(insulin_df: pd.DataFrame) -> tuple:
             - FASTRESN (float): Basal rate (U/hr) for basals
             - INSNMBOL (float): Normal bolus amount (U)
             - INSEXBOL (float): Extended bolus amount (U)
+        filtered_df (Optional: pd.DataFrame): Filtered insulin data. 
+            Specifically needed for netiob 24 hours data pull for every reference timestamps
         
     Returns:
         tuple: (pumphistory, pump_clock)
@@ -969,28 +994,26 @@ def get_pump_history(insulin_df: pd.DataFrame) -> tuple:
 
     # Create basal dataframe with computed duration column
     # Duration is time until next basal event
-    basal_with_duration = compute_basal_duration(main_basal_insulin_records)[
-        ['FADTC', 'FASTRESN', 'DURATION']
-    ]
+    basal_with_duration = compute_basal_duration(main_basal_insulin_records)[['FADTC', 'FASTRESN', 'DURATION']]
 
     # Define insulin delivery type categories
     basal_types = ['basal', 'basal_chunk']  # Types that represent basal delivery
     bolus_types = ['normal', 'extended']  # Types that represent bolus delivery
-
-    pumphistory = []
     
+    # determine which of the DFs to iterate. Pumphistory for prediction will be df
+    # while for netiob prediction will be filtered_df
+    df_iter = filtered_df if filtered_df is not None else df
+
+    pumphistory = []    
     # Process each insulin delivery record
-    for _, row in df.iterrows():
+    for _, row in df_iter.iterrows():
         # Convert timestamp to OpenAPS ISO format with 'Z'
-        ts = to_iso_z(row['FADTC'])
+        ts = datetime_to_zoned_iso(pd.to_datetime(row['FADTC']))
 
         # Process basal insulin events
         if row['FATEST'] == 'BASAL INSULIN' and row['INSSTYPE'] in basal_types:
             # Look up duration for this basal event
-            duration_row = basal_with_duration.loc[
-                basal_with_duration['FADTC'] == row['FADTC'], 
-                'DURATION'
-            ]
+            duration_row = basal_with_duration.loc[ basal_with_duration['FADTC'] == row['FADTC'], 'DURATION']
             duration = duration_row.iloc[0] if not duration_row.empty else 0
             # Convert NaN to 0 (last basal event has no duration)
             duration = 0 if np.isnan(duration) else duration
@@ -1012,32 +1035,52 @@ def get_pump_history(insulin_df: pd.DataFrame) -> tuple:
 
         # Process bolus insulin events
         elif row['FATEST'] == 'BOLUS INSULIN' and row['INSSTYPE'] in bolus_types:
-            # Extract normal bolus amount (handle None/'None' string values)
-            normal = float(row['INSNMBOL']) if pd.notna(row['INSNMBOL']) and row['INSNMBOL'] != 'None' else 0
+           
+            # FIX: Extract and validate bolus amounts
+            normal_amount = row.get('INSNMBOL', 0)
+            extended_amount = row.get('INSEXBOL', 0)
             
-            # Extract extended bolus amount (handle None/'None' string values)
-            extended = float(row['INSEXBOL']) if pd.notna(row['INSEXBOL']) and row['INSEXBOL'] != 'None' else 0
-            
-            # Create normal bolus event (always created, even if amount is 0)
-            pumphistory.append({
-                "timestamp": ts,
-                "_type": "Bolus",
-                "amount": normal,  # Actual insulin delivered (U)
-                "programmed": normal,  # Insulin programmed to deliver (U)
-                "unabsorbed": 0,  # Unabsorbed insulin from previous boluses
-                "duration": 0  # Instantaneous delivery (not extended over time)
-            })
+            # Convert NaN to 0 using pd.isna
+            normal_amount = 0 if pd.isna(normal_amount) else float(normal_amount)
+            extended_amount = 0 if pd.isna(extended_amount) else float(extended_amount)
             
+            # FIX: Only create bolus events for non-zero amounts
+            if row['INSSTYPE'] == 'normal':
+                # Only create event if normal bolus amount > 0
+                if normal_amount > 0:
+                    pumphistory.append({
+                        "timestamp": ts,
+                        "_type": "Bolus",
+                        "amount": normal_amount,
+                        "programmed": normal_amount,
+                        "unabsorbed": 0,
+                        "duration": 0
+                    })
+                    
             # Create separate event for extended bolus if present
-            if row['INSSTYPE'] == 'extended' and extended:
-                pumphistory.append({
-                    "timestamp": ts,
-                    "_type": "Bolus",
-                    "amount": extended,  # Extended bolus amount (U)
-                    "programmed": extended,
-                    "unabsorbed": 0,
-                    "duration": 0  # Simplified: treating as instantaneous
-                })
+            elif row['INSSTYPE'] == 'extended':
+                # Create normal component if > 0
+                if normal_amount > 0:
+                    pumphistory.append({
+                        "timestamp": ts,
+                        "_type": "Bolus",
+                        "amount": normal_amount,
+                        "programmed": normal_amount,
+                        "unabsorbed": 0,
+                        "duration": 0
+                    })
+                
+                # Create extended component if > 0
+                if extended_amount > 0:
+                    pumphistory.append({
+                        "timestamp": ts,
+                        "_type": "Bolus",
+                        "amount": extended_amount,
+                        "programmed": extended_amount,
+                        "unabsorbed": 0,
+                        "duration": 0  # Consider adding actual duration for extended
+                    })
+
     
     # Pump clock is timestamp of most recent event, or current time if no events
     if pumphistory:
@@ -1049,6 +1092,7 @@ def get_pump_history(insulin_df: pd.DataFrame) -> tuple:
     return pumphistory, pump_clock
 
 
+
 def get_glucose_and_clock(glucose_df: pd.DataFrame) -> tuple:
     """
     Build glucose status array with rate-of-change metrics.
