@@ -109,32 +109,27 @@ class OpenapsPredictor:
         if iobs is not None and not isinstance(iobs, list):
             raise TypeError("iobs must be a list of IOB objects or None")
         
+        optional_dfs = {
+            'basal_df': basal_df,
+            'bolus_df': bolus_df,
+            'carbs_df': carbs_df,
+            'cgm_df': cgm_df,
+            'profile_df': profile_df
+        }
+
+        # Validate input types
+        for df_name, df in optional_dfs.items():
+            if df is not None and not isinstance(df, pd.DataFrame):
+                raise TypeError(f"{df_name} must be a pandas DataFrame or None, got {type(df).__name__}")
+        
         # Store raw input dataframes
         self.profile_df = profile_df
         self.iobs = iobs
-        self.basal_df = basal_df
-        self.bolus_df = bolus_df
-        self.carbs_df = carbs_df
+        self.basal_df = basal_df if basal_df is not None else pd.DataFrame()
+        self.bolus_df = bolus_df if bolus_df is not None else pd.DataFrame()
+        self.carbs_df = carbs_df if carbs_df is not None else pd.DataFrame()
         self.cgm_df = cgm_df
-        
-        # Check for empty DataFrames and log warnings
-        empty_dfs = {
-            "basal_df": self.basal_df.empty,
-            "bolus_df": self.bolus_df.empty,
-            "carbs_df": self.carbs_df.empty,
-            "cgm_df": self.cgm_df.empty,
-            "profile_df": self.profile_df.empty,
-        }
-        
-        empty = [name for name, is_empty in empty_dfs.items() if is_empty]
-        if empty:
-            error_msg = (
-                f"The following DataFrames are empty: {', '.join(empty)}. "
-                f"Blood glucose prediction cannot function without core data."
-            )
-            logger.error(error_msg)
-            raise ValueError(error_msg)
-        
+            
         # Initialize attributes that will be set during processing
         self.data_processor = None
         self.proc_carbs = None
@@ -153,7 +148,7 @@ class OpenapsPredictor:
         try:
             # Create an instance of the CGMDataProcessor class to get all core prediction objects
             # This handles preprocessing and extraction of glucose data, pump history, profile, and autosens
-            self.data_processor = CGMDataProcessor(self.basal_df, self.bolus_df, self.carbs_df, self.cgm_df, profile_df=self.profile_df)
+            self.data_processor = CGMDataProcessor(basal_df=self.basal_df, bolus_df=self.bolus_df, carbs_df=self.carbs_df, cgm_df=self.cgm_df, profile_df=self.profile_df)
             
             # Validate that data_processor was created successfully
             if self.data_processor is None:
@@ -195,7 +190,7 @@ class OpenapsPredictor:
             # In that case, a self-defined baseline basal profile and autosensitivity ratio would be used.
             if self.iobs is None:
                 logger.info("Calculating net IOB from insulin and carb data")
-                iob_data = calculate_net_iob(self.basal_df, self.bolus_df, self.carbs_df, self.cgm_df, profile=self.profile_df, autosens=self.autosens)
+                iob_data = calculate_net_iob(basal_df=self.basal_df, bolus_df=self.bolus_df, cgm_df=self.cgm_df, profile=self.profile_df, autosens=self.autosens)
                 
                 # Validate IOB calculation result
                 if not iob_data:
